-- the time that is allowed to elapse from automatic dismount to remount. set to nil / 0 to disable this check.
MOUNTGATHERER_TIMEOUT = 15;

-- how much time to allow the user to loot
MOUNTGATHERER_TIME_TO_LOOT = 1;

-- how much time to allow the user to loot when using a "multiple" gathering skill (i.e. Mining).
MOUNTGATHERER_TIME_TO_LOOT_MULTIPLE = 3;


-- time between updates
MOUNTGATHERER_TIME_BETWEEN_UPDATES = 1;

MountGatherer_ShouldMountAt = nil;

MountGatherer_Saved_GameTooltip_Show = nil;
MountGatherer_LastShownTooltip = nil;
MountGatherer_TooltipStrings = {};

MountGatherer_HandledItem = nil;
MountGatherer_LastDismount = nil;
MountGatherer_HasMining = false;
MountGatherer_HasHerbalism = false;
MountGatherer_HasSkinning = false;

function MountGatherer_OnLoad()
	MountGatherer_Saved_GameTooltip_Show = GameTooltip.Show;
	GameTooltip.Show = MountGatherer_GameTooltip_Show;

	local f = MountGathererFrame;
	f:RegisterEvent("VARIABLES_LOADED");
	f:RegisterEvent("SPELLS_CHANGED");
	f:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF");
	f:RegisterEvent("UI_ERROR_MESSAGE");
	f:RegisterEvent("MAIL_CLOSED");
end

function MountGatherer_OnEvent(event)
	if ( event == "VARIABLES_LOADED" ) then
		MountGatherer_SetupGatheringSkills();
		MountGathererFrame:UnregisterEvent(event);
	end
	if ( event == "SPELLS_CHANGED" ) then
		MountGatherer_SetupGatheringSkills();
	end
	if ( event == "CHAT_MSG_SPELL_SELF_BUFF" ) then
		MountGatherer_HandlePerformChat(arg1);
	end
	if ( event == "MAIL_CLOSED" ) then
		MountGatherer_Mount();
	end
	if ( event == "UI_ERROR_MESSAGE" ) then
		if ( arg1 == SPELL_FAILED_NOT_MOUNTED ) then
			MountGatherer_FixMount();
		end
	end
end

MountGatherer_TooltipTextLeftNames = {
	[1] = "GameTooltipTextLeft1",
	[2] = "GameTooltipTextLeft2",
	[3] = "GameTooltipTextLeft3",
	[4] = "GameTooltipTextLeft4",
};

function MountGatherer_GameTooltip_Show(tooltip)
	if ( tooltip == GameTooltip ) then
		local obj = nil;
		for k, v in MountGatherer_TooltipTextLeftNames do
			obj = getglobal(v);
			if ( obj ) then
				MountGatherer_TooltipStrings[k] = obj:GetText();
			end
		end
		MountGatherer_LastShownTooltip = GetTime();
	end
	MountGatherer_Saved_GameTooltip_Show(tooltip);
end

function MountGatherer_HandlePerformed(action)
	if ( not action ) then
		return false;
	end
	local ok = false;
	for k, v in MountGatherer_Chat_SkillNames do
		if ( v == action ) then
			ok = true;
		end
	end
	if ( not ok ) then
		return false;
	end
	if ( MOUNTGATHERER_TIMEOUT ) and ( MOUNTGATHERER_TIMEOUT > 0 ) then
		if ( not MountGatherer_LastDismount ) or ( GetTime() - MountGatherer_LastDismount > MOUNTGATHERER_TIMEOUT ) then
			return false;
		end
	end
	
	-- fix for mining
	local addTime = MOUNTGATHERER_TIME_TO_LOOT;
	for k, v in MountGatherer_SkillNames_Multiple do
		if ( v == action ) then
			addTime = MOUNTGATHERER_TIME_TO_LOOT_MULTIPLE;
			break;
		end
	end
	MountGatherer_LastDismount = MountGatherer_LastDismount + addTime;
	MountGatherer_ShouldMountAt = GetTime() + addTime;
	MountGathererFrame:Show();
	return true;
end

function MountGatherer_HandlePerformChat(msg)
	local pattern = MountGatherer_Chat_PerformPattern.pattern;
	local index = MountGatherer_Chat_PerformPattern.actionIndex;
	for p1, p2, p3 in string.gfind(msg, pattern) do
		if ( index == 1 ) then
			MountGatherer_HandlePerformed(p1);
		elseif ( index == 2 ) then
			MountGatherer_HandlePerformed(p2);
		elseif ( index == 3 ) then
			MountGatherer_HandlePerformed(p3);
		end
	end
end


function MountGatherer_SetupGatheringSkills()
	local spellName = nil;
	MountGatherer_HasMining = false;
	MountGatherer_HasHerbalism = false;
	MountGatherer_HasSkinning = false;
	for i = 1, 150 do
		spellName = GetSpellName(i, "spell");
		if ( not spellName ) then
			break;
		end
		if ( spellName == MountGatherer_Skill_Mining ) then
			MountGatherer_HasMining = true;
		end
		if ( spellName == MountGatherer_Skill_Herbalism ) then
			MountGatherer_HasHerbalism = true;
		end
		if ( spellName == MountGatherer_Skill_Skinning ) then
			MountGatherer_HasSkinning = true;
		end
	end
	MountGatherer_SetupDismountableStrings();
end

function MountGatherer_SetupDismountableStrings()
	MountGatherer_DismountableStrings = {};
	if ( MountGatherer_HasMining ) then
		for k, v in MountGatherer_DismountableStringsMining do
			table.insert(MountGatherer_DismountableStrings, v);
		end
	end
	if ( MountGatherer_HasHerbalism ) then
		for k, v in MountGatherer_DismountableStringsHerbalism do
			table.insert(MountGatherer_DismountableStrings, v);
		end
	end
	for k, v in MountGatherer_DismountableStringsOther do
		table.insert(MountGatherer_DismountableStrings, v);
	end
	table.insert(MountGatherer_DismountableStrings, MountGatherer_DismountableStringsMailbox);
end

function MountGatherer_FixMount()
	local curTime = GetTime();
	if ( not MountGatherer_LastShownTooltip ) or ( curTime - MountGatherer_LastShownTooltip > 1 ) then
		return false;
	end
	MountGatherer_LastShownTooltip = nil;
	return MountGatherer_Dismount();
end

function MountGatherer_Dismount()
	local mountBuffPos = AutoMount_GetMountBuffPosition();
	if ( mountBuffPos > -1 ) then
		local obj = nil;
		if ( MountGatherer_HasSkinning ) then
			local msg = nil;
			for i = 3, 4 do
				msg = MountGatherer_TooltipStrings[i];
				if ( msg ) and ( strlen(msg) > 0 ) then
					for k, v in MountGatherer_DismountableStringsSkinning do
						MountGatherer_HandledItem = v;
						CancelPlayerBuff(mountBuffPos);
						MountGatherer_LastDismount = GetTime();
						return true;
					end
				end
			end
		end
		local msg = MountGatherer_TooltipStrings[1];
		if ( msg ) and ( strlen(msg) > 0 ) then
			for k, v in MountGatherer_DismountableStrings do
				if ( string.find(msg, v) ) then
					MountGatherer_HandledItem = v;
					CancelPlayerBuff(mountBuffPos);
					MountGatherer_LastDismount = GetTime();
					return true;
				end
			end
		end
	end
	return false;
end

function MountGatherer_Mount()
	if ( PlayerFrame.inCombat ) then
		return false;
	end
	local bag, slot = AutoMount_GetMountItemBagSlot();
	if ( not bag ) or ( not slot ) then
		return false;
	end
	UseContainerItem(bag, slot);
	return true;
end


MountGatherer_LastUpdate = 0;

function MountGatherer_OnUpdate(arg1)
	local curTime = GetTime();
	local diff = curTime - MountGatherer_LastUpdate;
	if ( diff < MOUNTGATHERER_TIME_BETWEEN_UPDATES ) then
		return false;
	end
	MountGatherer_LastUpdate = curTime;
	if ( MountGatherer_ShouldMountAt ) then
		if ( not LootFrame:IsVisible() ) then
			if ( curTime >= MountGatherer_ShouldMountAt ) then
				MountGatherer_Mount();
				MountGatherer_ShouldMountAt = nil;
				MountGathererFrame:Hide();
			end
		end
	else
		MountGathererFrame:Hide();
	end
end